@@ -33,8 +33,8 @@ public class ColorSelection extends VBox {
 	}
 	
 	protected void noopInit() {
-		if (-1 == handle) {
-			handle = gtk_color_selection_new ();
+		if (-1 == getHandle()) {
+			setHandle(gtk_color_selection_new ());
 			initializeEventHandlers();
 		}
 	}
@@ -43,14 +43,15 @@ public class ColorSelection extends VBox {
 	 * construct a ColorSelection from a handle to a native peer.
 	 */
 	public ColorSelection(int handle) {
-	    super(handle);
+	    setHandle(handle);
+	    initializeEventHandlers();
 	}
 
 	/**
 	 * Sets the widget to have an opacity control
 	 */
 	public void setHasOpacity(boolean hasOpacity) {
-		gtk_color_selection_set_has_opacity_control(handle, hasOpacity);
+		gtk_color_selection_set_has_opacity_control(getHandle(), hasOpacity);
 	}
 
 	/**
@@ -59,7 +60,7 @@ public class ColorSelection extends VBox {
 	 * @param color 
 	 */
 	public void setCurrentColor(Color color) {
-		gtk_color_selection_set_current_color(handle, color.getHandle());
+		gtk_color_selection_set_current_color(getHandle(), color.getHandle());
 	}
 
 	/**
@@ -67,7 +68,7 @@ public class ColorSelection extends VBox {
 	 * @return the set colour
 	 */
 	public Color getCurrentColor() {
-	    int colorHandle = gtk_color_selection_get_current_color(handle);
+	    int colorHandle = gtk_color_selection_get_current_color(getHandle());
 	    return new Color(colorHandle);
 	}
 	
@@ -76,7 +77,7 @@ public class ColorSelection extends VBox {
 	 * @param hasOpacity true if this widget can set the opacity.
 	 */
 	public void setHasOpacityControl(boolean hasOpacity) {
-		gtk_color_selection_set_has_opacity_control(handle, hasOpacity);
+		gtk_color_selection_set_has_opacity_control(getHandle(), hasOpacity);
 	}
 	
 	/**
@@ -84,7 +85,7 @@ public class ColorSelection extends VBox {
 	 * @return true if the widget has the opacity control.
 	 */
 	public boolean getHasOpacityControl() {
-		return gtk_color_selection_get_has_opacity_control(handle);
+		return gtk_color_selection_get_has_opacity_control(getHandle());
 	}
 	
 	/**
@@ -92,7 +93,7 @@ public class ColorSelection extends VBox {
 	 * @param hasPalette true if the palette should be visible.
 	 */
 	public void setHasPalette(boolean hasPalette) {
-		gtk_color_selection_set_has_palette(handle, hasPalette);
+		gtk_color_selection_set_has_palette(getHandle(), hasPalette);
 	}
 	
 	/**
@@ -100,7 +101,7 @@ public class ColorSelection extends VBox {
 	 * @return true if the palette is visible.
 	 */
 	public boolean getHasPalette() {
-		return gtk_color_selection_get_has_palette(handle);
+		return gtk_color_selection_get_has_palette(getHandle());
 	}
 	
 	/**
@@ -110,7 +111,7 @@ public class ColorSelection extends VBox {
 	 * @param alpha An integer between 0 and 65535.
 	 */
 	public void setCurrentAlpha(int alpha) {
-		gtk_color_selection_set_current_alpha(handle, alpha);
+		gtk_color_selection_set_current_alpha(getHandle(), alpha);
 	}
 	
 	/**
@@ -118,28 +119,28 @@ public class ColorSelection extends VBox {
 	 * @return An integer between 0 and 65535.
 	 */
 	public int getCurrentAlpha() {
-		return gtk_color_selection_get_current_alpha(handle);
+		return gtk_color_selection_get_current_alpha(getHandle());
 	}
 	
 	/**
 	 * Sets the previous alpha value.
 	 */
 	public void setPreviousAlpha(int alpha) {
-		gtk_color_selection_set_previous_alpha(handle, alpha);
+		gtk_color_selection_set_previous_alpha(getHandle(), alpha);
 	}
 	
 	/**
 	 * Returns the previous alpha value.
 	 */
 	public int getPreviousAlpha() {
-		return gtk_color_selection_get_previous_alpha(handle);
+		return gtk_color_selection_get_previous_alpha(getHandle());
 	}
 	
 	/**
 	 * Sets the previous color value.
 	 */
 	public void setPreviousColor(Color color) {
-		gtk_color_selection_set_previous_color(handle, color.getHandle());
+		gtk_color_selection_set_previous_color(getHandle(), color.getHandle());
 	}
 	
 	/**
@@ -147,7 +148,7 @@ public class ColorSelection extends VBox {
 	 */
 	public Color getPreviousColor() {
 		int [] color = new int[1];
-		gtk_color_selection_get_previous_color(handle, color);
+		gtk_color_selection_get_previous_color(getHandle(), color);
 		return new org.gnu.gdk.Color(color[0]);
 	}
 	
@@ -212,9 +213,15 @@ public class ColorSelection extends VBox {
 		fireColorSelectionEvent(new ColorSelectionEvent(this, ColorSelectionEvent.Type.COLOR_CHANGED));
 	}
 
+	/** Are events initialized? */
+	private boolean eventsInitialized = false;
+
 	protected void initializeEventHandlers() {
-		super.initializeEventHandlers();
-		evtMap.initialize(this);
+		if (!eventsInitialized) {
+			super.initializeEventHandlers();
+			evtMap.initialize(this);
+			eventsInitialized = true;
+		}
 	}
 
 	public Class getEventListenerClass(String signal) {
